# ***** BEGIN LICENSE BLOCK *****
# * Copyright (C) 2007  Alexey Gladkov <legion@altlinux.org>
# *
# * This program is free software; you can redistribute it and/or modify
# * it under the terms of the GNU General Public License as published by
# * the Free Software Foundation; either version 2 of the License, or
# * (at your option) any later version.
# *
# * This program is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# * GNU General Public License for more details.
# *
# * You should have received a copy of the GNU General Public License
# * along with this program; if not, write to the Free Software
# * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
# ***** END LICENSE BLOCK *****

set_bootable() {
	local bootdev="$1" disk= partition= i=0

	while read line; do
		case "$line" in
			Disk*)
				i=0
				disk="$(printf %s\\n "$line" |sed -ne 's/^Disk \([^:]\+\):.*/\1/p')"
				;;
			/dev/*)
				i="$(($i+1))"
				partition="${line%% *}"
				if [ "$bootdev" = "$partition" -a -n "$disk" ]; then
					debug "set_bootable(): set bootable flag on '$partition' partittion (dev='$disk', num='$i')"
					sfdisk "$disk" -A $i
					break
				fi
				;;
		esac
	done < "$fdisk_out"
}

set_bootable_raid() {
	[ -n "$raidboot" ] || return 0
	local d

	md-list "${raidboot#/dev/}" |
	while read d; do
		set_bootable "$d"
	done
}

bootable_flag() {
	local bootdev="$1"

	# Check raid
	[ -z "$raidtypes" ] || set_bootable_raid

	# Don't set bootable flag if some partitions already marked as bootable.
	partitions_list |grep "^$bootdev" |cut -d\  -f1,2 |grep -qs '[[:space:]]\*$' &&
		return 0

	if grep -qs "^Disk $bootdev:" "$fdisk_out"; then
		[ "$(fdisk -l "$bootdev" 2>/dev/null |grep '^/dev' |wc -l)" -gt 0 ] ||
			return 0

		sfdisk "$bootdev" -A 1
	else
		set_bootable "$bootdev"
	fi
}
