# ***** BEGIN LICENSE BLOCK *****
# * Copyright (C) 2007  Alexey Gladkov <legion@altlinux.org>
# *
# * This program is free software; you can redistribute it and/or modify
# * it under the terms of the GNU General Public License as published by
# * the Free Software Foundation; either version 2 of the License, or
# * (at your option) any later version.
# *
# * This program is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# * GNU General Public License for more details.
# *
# * You should have received a copy of the GNU General Public License
# * along with this program; if not, write to the Free Software
# * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
# ***** END LICENSE BLOCK *****

###
### Functions to work with lilo.conf
###

device_exists() {
	[ ! -e "$1" ] || return 0
	error "$1: device not found"
	return 1
}

cmdline_params() {
	local vga= args= new_args=

	vga="$(sed -rne 's|^(.* )?vga=([^ ]+)( .*)?$|\2|p' /proc/cmdline)"
	[ -z "$vga" ] || replace_global 'vga' "vga=\"$vga\""

	for p in $(cat /proc/cmdline); do
		case "$p" in
			# ACPI/APM
			acpi=*|noapic|acpi_*|nolapic*|lapic*|no_timer_check|pnpacpi=*|apm=*) ;;

			# [LIBATA] DMA control
			libata.*) ;;

			# [MOUSE]
			mousedev.*) ;;

			# [HW] (E)IDE subsystem
			# See Documentation/ide.txt
			ide=*|idebus=*) ;;

			# [BUGS=X86-32]
			mem=*|nmi_watchdog=*|no387|notsc) ;;

			# [PCI] various PCI subsystem options
			pci=*) ;;

			# Misc
			idle=*|irqfixup|irqpoll|clocksource=*|console=*|splash=*) ;;

			*) continue ;;
		esac
		new_args="$new_args $p"
	done
	args="$(show_global_param 'append')"
	args="$args${new_args:+ $new_args}"
	args="${args# }"

	[ -z "$args" ] ||
		replace_global 'append' "append=\"$args\""

	! readlink -e '/boot/splash/message' >/dev/null ||
		replace_global 'message' "message=\"/boot/splash/message\""

	commit "$globals"
}

update_rootdev() {
	local cnf="$1" dev mpoint dummy
	while read dev mpoint dummy; do
		[ "$mpoint" = "/" ] || continue
		sed -i -e "s,^root=.*,root=\"$dev\"," "$cnf"
		break
	done < /etc/fstab
}

convert_bootdev() {
	local devname="$1" cnf="$2" bootdev= boot_uuid=
	debug "convert_bootdev(): start"

	bootdev="$devname"
	if [ "$bootdev" = "${bootdev#/dev/md}" ]; then
		if boot_uuid="UUID=$(blkid -c /dev/null -w /dev/null -o value -s UUID "$devname" 2>/dev/null)"; then
			bootdev="$boot_uuid"
		fi
	fi

	debug "convert_bootdev(): device='$devname', bootdev='$bootdev'"
	sed -i -e "s,^boot=.*,boot=\"$bootdev\"," "$cnf"
}

write_liloconf() {
	tmpconf="$1"; shift
	newconf="$1"; shift

	local bootdev="$(sed -rne 's,^boot=\"?([^\"]+)\"?,\1,p' "$tmpconf")"
	if [ -z "$bootdev" ]; then
		error "'boot=' not found"
		return 1
	fi

	[ "$bootdev" != "${bootdev#/}" ] || bootdev="/dev/$bootdev"
	device_exists "$bootdev" || return 1

	# Convert boot device to UUID
	convert_bootdev "$bootdev" "$tmpconf" || return 1

	# Test lilo.conf
	if ! out="$(lilo -t -H -C "$tmpconf" </dev/null 2>&1 >/dev/null)"; then
		error "Lilo test: $out"
		openclose_cdrom
		return 1
	fi
	debug "write_liloconf(): lilo test passed"

	# Set bootable flag
	bootable_flag "$bootdev"

	# Write lilo.conf
	[ ! -f "$newconf" ] || mv -f -- "$newconf" "$newconf".save
	mv -f -- "$tmpconf" "$newconf"

	# Real install bootloader
	lilo -H </dev/null >/dev/null 2>&1
	openclose_cdrom
}

generate_liloconf() {
	tmpconf="$1"; shift

	install -m600 "$globals" "$tmpconf"

	while read -r t s; do
		[ -f "$sections/$s" ] || continue
		printf \\n
		format_section "$sections/$s"
	done < "$order" >> "$tmpconf"
}

normalize_liloconf() {
	local keyword dummy sysconf= tmpconf= conflist=
	sysconf="$1"; shift
	tmpconf="$1"; shift

	if [ -s "$sysconf" ]; then
		grep -qs '^boot=' "$sysconf" &&
			conflist="$sysconf" ||
			conflist="$tmpconf $sysconf"
	else
		conflist="$tmpconf"
	fi

	sed -r	-e 's#[[:space:]]+# #g' \
		-e 's#^[[:space:]]*([^=]+=)(.*)#\1 \2#' \
		$conflist |
	while read -r keyword dummy; do
		[ -n "$keyword" ] || continue
		if [ -z "${keyword##*=}" -a -n "$dummy" ]; then
			[ -z "${dummy%%\"*}" ] || dummy="\"$dummy"
			[ -z "${dummy##*\"}" ] || dummy="$dummy\""
		fi
		keyword="$(printf %s\\n "$keyword" |tr '[A-Z]' '[a-z]')"
		printf %s\\n "$keyword$dummy"
	done
}
