; ***** BEGIN LICENSE BLOCK *****
; * Copyright (C) 2007  Alexey Gladkov <legion@altlinux.org>
; *
; * This program is free software; you can redistribute it and/or modify
; * it under the terms of the GNU General Public License as published by
; * the Free Software Foundation; either version 2 of the License, or
; * (at your option) any later version.
; *
; * This program is distributed in the hope that it will be useful,
; * but WITHOUT ANY WARRANTY; without even the implied warranty of
; * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; * GNU General Public License for more details.
; *
; * You should have received a copy of the GNU General Public License
; * along with this program; if not, write to the Free Software
; * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
; ***** END LICENSE BLOCK *****

(document:surround "/std/frame")
(document:surround "/std/functions")
(document:envelop with-translation _ "alterator-lilo")

(document:envelop with-attributes (password1 password2))
(document:envelop with-container-presentations ((password-box '/root/passwd)))

(define (get-global-params)
	(let ((info (woo-read-first '/lilo/globals)))
		(global-boot text (woo-get-option info 'boot ""))
		(global-map text (woo-get-option info 'map ""))
		(global-timeout text (woo-get-option info 'timeout ""))
		(global-append text (woo-get-option info 'append ""))
		(config-only state (woo-get-option info 'configonly #f))
		(mandatory state (woo-get-option info 'mandatory #f))
		(restricted state (woo-get-option info 'restricted #f))
		(insecure state (not (or (mandatory state) (restricted state))))

		;;; XXXlegion: Insecure!!! FIXME!!!
		(pbox password1 (woo-get-option info 'password ""))
		(pbox password2 (woo-get-option info 'password ""))

		(global-install current (string-list-index	(woo-get-option info 'install "text")
								(map	(lambda(x) (car (vector-ref x 0)))
									(global-install rows))))

		(global-raid-extra-boot current (or (string-list-index (woo-get-option info 'raid-extra-boot "")
								       (map	(lambda(x) (car (vector-ref x 0)))
										(global-raid-extra-boot rows)))
						    0))))
(define (real-write-globals url)
	(let ((passwd1 (pbox password1)) (passwd2 (pbox password2)))
		(woo-write/constraints url
				       'target   "globals"
				       'boot     (global-boot text)
				       'map      (global-map text)
				       'append   (global-append text)
				       'timeout  (global-timeout text)
				       'install  (global-install text)
				       'extraboot (global-raid-extra-boot current-text)
				       'password1 passwd1
				       'password2 passwd2
				       'method   (or (and (mandatory state) "mandatory")
						     (and (restricted state) "restricted")
						     "insecure")
				       'confonly (config-only state))))
(define (write-globals)
	(woo-catch/message (thunk (real-write-globals "/lilo")
				  (real-write-globals "/autoinstall/lilo"))))
(define (write-all)
	(if (or (not (skip-check state)) (not (global 'frame:next)))
	    (and	(write-globals)
			(woo-catch/message (thunk (woo-try "commit" "/lilo")
						  (woo-try "commit" "/autoinstall/lilo"))))
	    #t))

(define (write-reset)
	(woo-catch/message (thunk (woo-try "reset" "/lilo" 'part "globals")
				  (woo-try "reset" "/autoinstall/lilo" 'part "globals")))
	(get-global-params))

(define (change url)
	(woo-catch/message (thunk (write-globals)))
	(frame:replace url))

(define (skip-bootloader . st)
	(normal-button		activity st)
	(perimage-button	activity st)
	(otherimage-button	activity st)
	(vsource-button		activity st)
	(bootloader-options	activity st))

(hbox
	align 'center
	(document:id normal-button (button (_ "Normal mode") (when clicked (change "/lilo"))))
	(button (_ "Global options") activity #f)
	(document:id perimage-button (button (_ "Operation systems") (when clicked (change "/lilo/extra-sections"))))
	(document:id otherimage-button (button (_ "Other loaders") (when clicked (change "/lilo/extra-other")))))

(groupbox ""
	(gridbox columns "2;98"
		(document:id skip-check
			(checkbox "" state #f activity (->bool (global 'frame:next)) (when toggled (skip-bootloader (not (skip-check state))))))
		(label (_ "Skip bootloader installation.") activity (->bool (global 'frame:next)))

		(document:id config-only
			(checkbox "" state #f))
		(label (_ "Write lilo.conf without boot loader installation."))
	)

	(document:id bootloader-options (vbox

	(hbox align 'left
		(document:id discard-button (button (_ "Discard changes") (when clicked (write-reset))))
		(document:id vsource-button (button (_ "View source") (when clicked (change "/lilo/view-source"))))
	)

	(gridbox columns "10;30;30;30"
		(spacer)	(label (_ "Boot:")    align 'right) 	(document:id global-boot (edit ""))				(spacer)
		(spacer)	(label (_ "Map:")     align 'right)	(document:id global-map  (edit ""))				(spacer)
		(spacer)	(label (_ "Append:")  align 'right)	(document:id global-append  (edit ""))				(spacer)
		(spacer)	(label (_ "Timeout:") align 'right)	(document:id global-timeout (edit ""))				(spacer)
		(spacer)	(label (_ "Install:") align 'right)	(document:id global-install (combobox rows '("text" "menu")))	(spacer)
		(spacer)	(label (_ "Raid-extra-boot:") align 'right)
				(document:id global-raid-extra-boot (combobox rows '("" "none" "auto" "mbr" "mbr-only")
									      current 0
									      alterability #t))
				(spacer)
	)
	(groupbox (_ "Security")
		(gridbox columns "2;98"
			(document:id insecure   (radio "" state #t))	(label (_ "Do not use password") text-wrap #t)
			(document:id mandatory  (radio "" state #f))	(label (_ "Ask password on every system boot") text-wrap #t)
			(document:id restricted (radio "" state #f))	(label (_ "Ask a password only when additional parameters are specified") text-wrap #t))

		(vbox	max-width 250 max-height 100
			(document:id pbox (password-box)))
	)
	))
)

(or
	(global 'frame:next)
	(hbox
		(button (_ "Apply") (when clicked (write-all)))
		(button (_ "Quit" ) (when clicked (document:end)))))
(and
	(global 'frame:next)
	(frame:on-next (thunk (or (write-all) 'cancel))))

(document:root (when loaded
			(and (global 'frame:next) (frame:back-activity #f))
			(get-global-params)))
