; ***** BEGIN LICENSE BLOCK *****
; * Copyright (C) 2007  Alexey Gladkov <legion@altlinux.org>
; *
; * This program is free software; you can redistribute it and/or modify
; * it under the terms of the GNU General Public License as published by
; * the Free Software Foundation; either version 2 of the License, or
; * (at your option) any later version.
; *
; * This program is distributed in the hope that it will be useful,
; * but WITHOUT ANY WARRANTY; without even the implied warranty of
; * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; * GNU General Public License for more details.
; *
; * You should have received a copy of the GNU General Public License
; * along with this program; if not, write to the Free Software
; * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
; ***** END LICENSE BLOCK *****

(document:surround "/std/frame")
(document:envelop with-translation _ "alterator-lilo")

(define (change url)
	(frame:replace url))

(define (info-changed)
	(button-apply activity #t))

(define sections (make-cell '()))
(define (make-view cmd) (woo-get-option cmd 'name))

(define (get-section-params)
	(button-apply activity #f)
	(let ((info (woo-read-first "/lilo" 'part "sections" 'type "image" 'section (sects text))))
		(section-label text (woo-get-option info 'label ""))
		(section-image text (woo-get-option info 'image ""))
		(section-initrd text (woo-get-option info 'initrd ""))
		(section-root text (woo-get-option info 'root ""))
		(section-append text (woo-get-option info 'addappend ""))))

(define (update-sections-list)
	(sects rows (map make-view (cell-ref sections)))
	(sects current (list-index (lambda(x) (woo-get-option x 'default #f)) (cell-ref sections))))

(define (init-sections)
	(cell-set! sections (woo-catch/message (thunk (woo-list "/lilo" 'part "sections" 'type "image"))))
	(update-sections-list))

(define (reload-sections)
	(init-sections)
	(get-section-params))

(define (add-section)
	(cell-set! sections (append
				    (cell-ref sections)
				    `(("" name ,(_ "<UNDEFINED>") default #f))))
	(update-sections-list)
	(sects active (- (length (sects rows)) 1))
	(get-section-params))

(define (remove-section)
	(let ((sects-current (sects current)))
		(and (not (string= (sects text) (_ "<UNDEFINED>")))
		     (woo-catch/message (thunk (woo-try "remove-section" "/lilo" 'label (sects text))
					       (woo-try "remove-section" "/autoinstall/lilo" 'label (sects text))))
		     (cell-set! sections (list-remove (cell-ref sections) (sects active)))
		     (init-sections)
		     (sects active 0)
		     (sects current (if (< (length (cell-ref sections)) sects-current) 0 sects-current))
		     (get-section-params))))

(define (real-write-section url)
	(woo-try
	 	(if (string= (sects text) (_ "<UNDEFINED>"))
		    "new-section"
		    "update-section")
		url
		'type     "image"
		'oldlabel (if (string= (sects text) (_ "<UNDEFINED>")) "" (sects text))
		'label    (section-label text)
		'image    (section-image text)
		'initrd   (section-initrd text)
		'root     (section-root text)
		'append   (section-append text)))

(define (write-section)
	(let ((sects-current (sects current)) (sects-active (sects active)))
	    (and (woo-catch/message (thunk (real-write-section "/lilo")
					   (real-write-section "/autoinstall/lilo")))
		 (button-apply activity #f)
		 (init-sections)
		 (sects current sects-current)
		 (sects active sects-active))))

(define (write-all)
	(woo-catch/message (thunk (woo-try "commit" "/lilo")
				  (woo-try "commit" "/autoinstall/lilo"))))

(define (write-reset)
	(woo-catch/message (thunk (woo-try "reset" "/lilo" 'part "sections")
				  (woo-try "reset" "/autoinstall/lilo" 'part "sections")))
	(reload-sections))

(hbox
	align 'center
	(button (_ "Normal mode") (when clicked (change "/lilo")))
	(button (_ "Global options") (when clicked (change "/lilo/extra-general")))
	(button (_ "Operation systems") activity #f)
	(button (_ "Other loaders") (when clicked (change "/lilo/extra-other"))))

(groupbox ""
	(hbox
		align 'left
		(button (_ "Set default") (when clicked
							(and
								(sects current (sects active))
								(woo-catch/message (thunk (woo-write "/lilo"
												     'target "default"
												     'default (sects text))
											  (woo-write "/autoinstall/lilo"
												     'target "default"
												     'default (sects text)))))))
		(button (_ "Add") (when clicked (add-section)))
		(button (_ "Remove") (when clicked (remove-section)))
		(button (_ "Discard changes") (when clicked (write-reset))))

	(hbox
		(document:id sects (radiolist (when selected (get-section-params))))

		(groupbox (_ "Information")
			(gridbox columns "10;20;60;10"
				(spacer)	(label (_ "Label:")  align 'right)	(document:id section-label  (edit "" (when changed (info-changed))))	(spacer)
				(spacer)	(label (_ "Image:")  align 'right)	(document:id section-image  (edit "" (when changed (info-changed))))	(spacer)
				(spacer)	(label (_ "Initrd:") align 'right)	(document:id section-initrd (edit "" (when changed (info-changed))))	(spacer)
				(spacer)	(label (_ "Root:")   align 'right)	(document:id section-root   (edit "" (when changed (info-changed))))	(spacer)
				(spacer)	(label (_ "Append:") align 'right)	(document:id section-append (edit "" (when changed (info-changed))))	(spacer))
			(hbox
				align 'center
				(document:id button-apply (button (_ "Apply") activity #f (when clicked (write-section))))
				(button (_ "Discard") (when clicked (get-section-params))))
		)
	)
)

(or
	(global 'frame:next)
	(hbox
		(button (_ "Apply") (when clicked (write-all)))
		(button (_ "Quit" ) (when clicked (document:end)))))
(and
	(global 'frame:next)
	(frame:on-next (thunk (or (write-all) 'cancel))))

(document:root (when loaded
			(and (global 'frame:next) (frame:back-activity #f))
			(reload-sections)))
