; ***** BEGIN LICENSE BLOCK *****
; * Copyright (C) 2007  Alexey Gladkov <legion@altlinux.org>
; *
; * This program is free software; you can redistribute it and/or modify
; * it under the terms of the GNU General Public License as published by
; * the Free Software Foundation; either version 2 of the License, or
; * (at your option) any later version.
; *
; * This program is distributed in the hope that it will be useful,
; * but WITHOUT ANY WARRANTY; without even the implied warranty of
; * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; * GNU General Public License for more details.
; *
; * You should have received a copy of the GNU General Public License
; * along with this program; if not, write to the Free Software
; * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
; ***** END LICENSE BLOCK *****

(document:surround "/std/frame")
(document:insert "/std/functions")
(document:envelop with-translation _ "alterator-lilo")

(define full-targets (make-cell '()))
(define short-targets (make-cell '()))

(define (disk? cmd)
	(woo-get-option cmd 'disk #f))

(define (boot-current field)
	(let ((cur (bootlist current)))
		(woo-get-option
			(list-ref (if (parts state) (cell-ref full-targets) (cell-ref short-targets)) cur)
			field)))

(define (write-boot)
	(and (woo-write "/lilo" 'target "boot" 'devname (boot-current 'name))
	     (woo-write "/autoinstall/lilo" 'target "boot" 'devname (boot-current 'name))))

(define (write-all)
	(woo-catch/message (thunk (write-boot)
				  (woo-try "commit" "/lilo")
				  (woo-try "commit" "/autoinstall/lilo"))))

(define (change url)
	(woo-catch/message (thunk (write-boot)))
	(frame:replace url))

(define (make-view cmd)
  (vector
	(woo-get-option cmd 'name)
	(woo-get-option cmd 'size)
        (if (woo-get-option cmd 'disk #f)
		(_ "Master Boot Record")
                (_ "First sector of partition"))
	""))

(define (make-rows lst)
	(map make-view lst))

(define (find-current)
	(or (string-list-index
		(woo-get-option (or (cond-car (filter (lambda(x) (woo-get-option x 'cur #f)) (cell-ref full-targets))) '()) 'name)
		(map (lambda(x) (woo-get-option x 'name)) (cell-ref full-targets)))
	0))

(define (restore-list)
	(let ((cur (find-current)))
		(if (>= cur (length (cell-ref short-targets)))
			(and (bootlist rows (make-rows (cell-ref full-targets))) (parts state #t))
			(bootlist rows (make-rows (cell-ref short-targets))))
		(bootlist current cur)))

(and (global 'frame:next)
    (label (_ "Boot loader is used to boot your ALTLinux system and it can be configured here. If ALTLinux is your sole operating system on this machine you should install the boot loader on Master Boot Record. Otherwise, please consult the documentation for available options.") text-wrap #t))

(gridbox columns "10;80;10"
	(spacer)
	(vbox
		(label (_ "Install boot loader on:"))
		(document:id bootlist (listbox
				columns 3
				max-height 200
				header (vector (_ "Device") (_ "Size") (_ "Destination"))))
		(hbox
			(document:id parts
			(checkbox (_ "Show partitions") state #f
				(when toggled
					(let ((cur (bootlist current)))
						(bootlist rows (make-rows
							(if (parts state)
								(cell-ref full-targets)
								(cell-ref short-targets))))
						(bootlist current
							(if (>= (length (bootlist rows)) cur) cur 0))))))
			(document:id expert-button (button (_ "Expert settings") (when clicked (change "/lilo/extra-general"))))))
	(spacer))

(or
	(global 'frame:next)
	(hbox
		(button (_ "Apply") (when clicked (write-all)))
		(button (_ "Quit" ) (when clicked (document:end)))))

(and
	(global 'frame:next)
	(frame:on-next (thunk (or (write-all) 'cancel))))

(document:root (when loaded
	(cell-set! full-targets (or (woo-catch/message (thunk (woo-list '/lilo))) '()))
	(cell-set! short-targets (filter disk? (cell-ref full-targets)))
	(restore-list)
	(expert-button (not (null? (bootlist rows))))

	(and (global 'frame:next)			
		(frame:back-activity #f)
		(frame:next-activity (not (null? (bootlist rows)))))
    )
)
